#include "memory.h"
#include "MyHash.h"

// | KEY |\0|VALUE|\0| KEY |\0|VALUE|\0\0 <- terminator

static char	szEmpty[] = "";

//-----------------------------------------------------------------------------
// Perform a lowercase comparison of strings.
int MyHash::StrCmpI( const char *str1, const char *str2 )
{
	do
	{
		if( ToLower(*str1) != ToLower(*str2) )
			return ToLower(*str2) - ToLower(*str1);
	}while( *str1++ && *str2++ );

	return 0;
}

//-----------------------------------------------------------------------------
// Compare strings
int MyHash::StrCmp( const char *str1, const char *str2 )
{
	do
	{
		if( *str1 != *str2 )
			return *str2 - *str1;
	}while( *str1++ && *str2++ );
	return 0;
}

//-----------------------------------------------------------------------------
//	@detail
//		Constracter
//	@param
//	@return
//-----------------------------------------------------------------------------
MyHash::MyHash()
{
	m_pszSeekPointer = NULL;
	m_pHeap = new MyVarMem;
}

MyHash::~MyHash()
{
	if( m_pHeap )
		delete m_pHeap;
}

//-----------------------------------------------------------------------------
//	@detail
//		Add record(key and value).
//	@param
//		cahr*	pszKey		Key
//		char*	pszValue	value
//	@return
//		Number of add bytes to heap.
//-----------------------------------------------------------------------------
int MyHash::Set( char* pszKey, char* pszValue )
{
	if( pszKey == NULL || pszValue == NULL )
		return 0;

	// If a key already exists, it will delete (it overwrites).

	if(SeekToKey( pszKey ) != NULL)
	{
		Del(pszKey);
	}

	// -- 
	int ret;
	int keylen = StrLen(pszKey) + 1;	// obtain null terminate.
	int valuelen = StrLen(pszValue) + 1;

	char* record = new char[keylen + valuelen];
	memset(record, 0x00,	keylen + valuelen);
	memcpy(record,			pszKey,		keylen);
	memcpy(record + keylen,	pszValue,	valuelen);
	
	ret = m_pHeap->Add(record, keylen + valuelen);

	delete[] record;
	return ret;
}

//-----------------------------------------------------------------------------
//	@detail
//		Delete record
//	@param
//		char* pszKey	Key to delete
//	@return
//		success	1
//		fail	0
//	@comment
//		Deletion is caused rearrangement of memory.
//		If possible don't do for speed
//-----------------------------------------------------------------------------
int MyHash::Del( char* pszKey )
{
	char* pTop = (char*)m_pHeap->GetBuffer();
	char* pDelKey = SeekToKey(pszKey);

	// There is no key.
	if( pDelKey == NULL )
	{
		return 0;
	}

	int nTotalLen = m_pHeap->GetLength();

	// When there are in a head.
	if(pTop == pDelKey)
	{
		MyVarMem* pHead = new MyVarMem;
		char* pNew = SeekNext();
		
		if( pNew != NULL )
		{
			pHead->Add(pNew, nTotalLen - (pNew - pTop));
		}
		delete m_pHeap;
		m_pHeap = pHead;
		SeekToFirst();
		return 1;
	}

	// When there are in the tail end.
	char* pNext = SeekNext();

	if( pNext == NULL )
	{
		MyVarMem* pHead = new MyVarMem;
		int nNewLen = pDelKey - pTop;
		pHead->Add( pTop, nNewLen );
		delete m_pHeap;
		m_pHeap = pHead;
		
		SeekToFirst();
		return 1;
	}

	// When there ara in the middle
	else
	{
		MyVarMem* pHead = new MyVarMem;
		
		int nHeadLen = pDelKey - pTop;
		pHead->Add(pTop, nHeadLen);

		int nTailLen = nTotalLen - (m_pszSeekPointer - pTop);
		pHead->Add(m_pszSeekPointer, nTailLen);

		delete m_pHeap;
		m_pHeap = pHead;
		
		SeekToFirst();
		return 1;
	}
}

//-----------------------------------------------------------------------------
//	@detail
//		Get value
//	@param
//		char*	pszKey	key
//	@return
//		 value
//-----------------------------------------------------------------------------
char* MyHash::GetValue( char* pszKey )
{
	char* p = (char*) m_pHeap->GetBuffer();

	p = SeekToKey(pszKey);
	if( p == NULL )
		return NULL;

	// Value ̓܂ňړ
	while(*p++ != '\0');
	
	return p;
}
//-----------------------------------------------------------------------------
//	@detail
//		Get value by the key without regard to case
//	@param
//		char*	pszKey	key without regard to case
//
//	@return
//		 l
//-----------------------------------------------------------------------------
char* MyHash::GetValueI(char *pszKey)
{
	char* p = (char*) m_pHeap->GetBuffer();

	p = SeekToKeyI(pszKey);
	if( p == NULL )
		return NULL;

	while(*p++ != '\0');
	
	return p;

}

//-----------------------------------------------------------------------------
//	@detail
//		Set seek pointer to a head.
//	@param
//	@return
//-----------------------------------------------------------------------------
char*	MyHash::SeekToFirst()
{
	m_pszSeekPointer = (char*)(m_pHeap->GetBuffer());
	return m_pszSeekPointer;
}

//-----------------------------------------------------------------------------
//	@detail
//		Seek pointer to the specified key.
//	@param
//		char*	pszKey	key
//	@return
//		If there is, return key.
//		fail return null.
//-----------------------------------------------------------------------------
char*	MyHash::SeekToKey(char* pszKey)
{
	SeekToFirst();
	char* p = (char*) m_pHeap->GetBuffer();

	if( p == NULL )
	{
		return NULL;
	}

	while(this->StrCmp(p, pszKey) != 0)
	{
		p = SeekNext();
		if( p == NULL )
			return NULL;
		
	}
	return (m_pszSeekPointer = p);
}
//-----------------------------------------------------------------------------
//	@detail
//		Seek pointer to the specified key.
//	@param
//		char*	pszKey	key
//				with out regard to case
//	@return
//		If there is, return key.
//		fail return null.
//-----------------------------------------------------------------------------
char*	MyHash::SeekToKeyI(char* pszKey)
{
	SeekToFirst();
	char* p = (char*) m_pHeap->GetBuffer();

	if( p == NULL )
	{
		return NULL;
	}

	while(StrCmpI(p, pszKey) != 0)
	{
		p = SeekNext();
		if( p == NULL )
			return NULL;
		
	}
	return (m_pszSeekPointer = p);
}

//-----------------------------------------------------------------------------
//	@detail
//		Return the current key.
//	@param
//		none
//	@return
//		pointer to key.
//-----------------------------------------------------------------------------
char*	MyHash::GetCurrentKey()
{
	return m_pszSeekPointer;
}

//-----------------------------------------------------------------------------
//	@detail
//		Return the current value.
//	@param
//		none
//	@return
//		pointer to value.
//-----------------------------------------------------------------------------
char*	MyHash::GetCurrentValue()
{
	if( m_pszSeekPointer == NULL )
	{
		return NULL;
	}
	char* p = m_pszSeekPointer;
	// Value ܂ňړ
	while(*p++ != '\0');

	return p;
}

//-----------------------------------------------------------------------------
//	@detail
//		Seek pointer to next key.
//	@param
//		none
//	@return
//		Return the next key.
//-----------------------------------------------------------------------------
char*	MyHash::SeekNext()
{
	if( m_pszSeekPointer == NULL )
	{
		return NULL;
	}
	char* p = m_pszSeekPointer;
		
	// Seek pointer to the current value.
	while(*p++ != '\0');
	
	// The value is ""
	if( p[0] == '\0' && p[1] == '\0' )
	{

		return NULL;
	}

	// Go the next.
	while(*p++ != '\0');

	// Return null if the end.
	if( *p == '\0' )
	{
		m_pszSeekPointer = p;
		return NULL;
	}

	return (m_pszSeekPointer = p);
}
/*
//-----------------------------------------------------------------------------
//	@detail
//		Delete all.
//	@param
//		noen
//	@return
//		none
//-----------------------------------------------------------------------------
void MyHash::DeleteAll()
{
	m_pHeap->Clear();
	m_pszSeekPointer = NULL;
}
*/

//-----------------------------------------------------------------------------
//	@detail
//		Get internal buffer
//	@param
//		noen
//	@return
//		none
//-----------------------------------------------------------------------------
void* MyHash::GetBuffer() const
{
	return m_pHeap->GetBuffer();
}
