/* =========================================================================
	This library is part of IshiSoft CEHTTP base class.
	------------------------------------------------------------------------
	author: yu-ishi@ya2.so-net.ne.jp
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#if !defined(AFX_MYHTTPSOCK_H__EAA73A6E_8C40_4F9B_8BC3_72CF9A94131F__INCLUDED_)
#define AFX_MYHTTPSOCK_H__EAA73A6E_8C40_4F9B_8BC3_72CF9A94131F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MySock.h"
#include "MyVarMem.h"
#include "MyHash.h"
#include "MyContentType.h"

//-----------------------------------------------
enum HTTPCMD
{
	HTTPUNKNOWN = 0,	// bad request
	HTTPHEAD,			// Implemented
	HTTPGET,			// Implemented
	HTTPPOST,
	/* Not Implemented
	HTTPPUT,
	HTTPDELETE,
	HTTPTRACE,
	HTTPOPTIONS,
	HTTPCONNECT,
	HTTPLINK,
	HTTPUNLINK
	*/
	HTTPNOTIMPLEMENTED
};

//-----------------------------------------------
// Server infomation.(pointers)
// inherit form MyComf class
//-----------------------------------------------
typedef struct tagServerInfo
{
	int		nPort;
	char*	pszSoftware;
	char*	pszDocRootName;
	char*	pszCgiRootName;
	char*	pszDefaultDocName;
	char*	pszAccessFileName;
	WCHAR*	pwszLogFile;
	int		nLogSize;
} SERVER_INFO, *LPSERVER_INFO;

//-----------------------------------------------
// CONTENT_TYPE
//-----------------------------------------------
typedef struct tagCONTENT_TYPE
{
	const char* pszExt;
	const char* pszMIME;
} CONTENT_TYPE, *LPCONTENT_TYPE;

//-----------------------------------------------
// HTTP_REQUEST
// The information client request is collected.
//-----------------------------------------------
typedef struct tagHTTP_REQUEST
{
	// Request line
	HTTPCMD			HttpCmd;				// GET, HEAD, POST,...
	char			szUrlPath[MAX_PATH];	// Url path
	char			szFilePath[MAX_PATH];	// File path of local file system
	char			szHttpVer[16];			// HTTP version of request
	
	WIN32_FIND_DATA	wfdFile;				// File information
	char			szFileTime[32];			// File time
	sockaddr_in*	psaRemoteHost;			// Remote host

} HTTP_REQUEST, *LPHTTP_REQUEST;

//-----------------------------------------------
// Relation with HTTPSTATUS and strings.
//-----------------------------------------------
typedef struct tagHTTP_STATUS
{
	int nStatus;
	const char* pszString;
} HTTP_STATUS, *LPHTTP_STATUS;

typedef int (*CEHTTP_CHX_EXTENTION)(MyHash*, MyVarMem*, MyVarMem*);
typedef int (*CEHTTP_DLL_EXTENTION)(const void*, int, const void*, int, const void*, int, void**);

//-----------------------------------------------
class MyHttpSock : public MySock  
{
public:
	MyHttpSock(SOCKET s);
	MyHttpSock();
	virtual ~MyHttpSock();

	LPSERVER_INFO	m_pServerInfo;	// Servers infomation
	MyHash			m_hashReqHeader;// Request header lines
	MyHash			m_hashEnv;		// Environment
	HTTP_REQUEST	m_hrHttpReq;	// HTTP_REQUEST
	MyVarMem		m_hReqHead;		// HTTP request header
	MyVarMem		m_hReqBody;		// HTTP request body

public:
	void	FreeMemory();
	int		SendResponse(int nHttpStatus, int bConnectionClose = 1);
	int		RecvRequest(int* pbConnectionClose);

protected:
	DWORD	m_dwRecvTimeOut;
protected:
	int		ExecuteDll(char* pwszFile, int bClose);
	int		ExecuteChx(char* tszFile, int bClose);
	int		ExecuteCGI(char* tszFile, int bClose);
	int		CheckBasicAuth(int nHttpStatus, const char* pszFilePath, const char* pszBAuthFile);
	int		CheckBasicAuthFile(int nHttpStatus, char* pszAuthFilePath);
	int		SendDefaultHttpResponse(int nHttpStatus, int bConnectionClose = 0, int nAddtionalInfo = 0);
	void	WorkMakeEnv();
	int		WorkMakePath(char* pszSrc, int nBufferSize = MAX_PATH);

	int		WorkParseRequestHeader(const char* pSrc);
	int		ParseRequestHeader( const char* pReq, char* szReqKey, char* szReqValue );
	int		WorkParseRequestLine(const char* pSrc );
	int		ParseRequestLine(const char* pReq, char* szCmd, char* szPath, char* szHttpVer);
private:
};

#endif // !defined(AFX_MYHTTPSOCK_H__EAA73A6E_8C40_4F9B_8BC3_72CF9A94131F__INCLUDED_)
