#include "stdafx.h"
#include "resource.h"

#include <stdio.h>
#include <stdlib.h>	// for PCHTTP
#include "MyHttpSock.h"
#include "httpd.h"
#include "MyConf.h"
#include "httphelper.h"

// global var.
SERVER_INFO	g_ServerInfo;
MyConf		g_conf;


#ifndef _WIN32_WCE
#define FNC_LOG_API	"fnCehttpLog"
#else
#define FNC_LOG_API	L"fnCehttpLog"
#endif

#define LOGAPIFILE L"\\cehttplog.dll"

#if defined(DEBUG)||defined(_DEBUG)
#define DBGPRINT(hWnd, nID, nStatus, p) \
	{WCHAR buf[256];wsprintf(buf, TEXT("%03u:%s"), nStatus, p);SetDlgItemText(hWnd, nID, buf);}
#else
#define DBGPRINT(hWnd, nID, nStatus, p);
#endif


//-----------------------------------------------------------------------------
//	@detail
//		The client thread invoked listen thread.
//	@param
//		THREADPACK
//	@return
//-----------------------------------------------------------------------------
void thAccept(LPVOID pvParams)
{
	
	// ------------------------------------------
	// Initialize thread
	// ------------------------------------------
	char szConnection[] = "connection";
	char szKeepAlive[] = "keep-alive";
	char szClose[] = "close";
	char szRemoteHost[16];
	
	char* pszCon = NULL;
	int bConnectionClose = 0;

	CHILD_THREAD* pChildThread = (CHILD_THREAD*)pvParams;
	MyHttpSock* s = new MyHttpSock();

	// Copy the remote host informations.
	s->m_hrHttpReq.psaRemoteHost = &pChildThread->saRemoteHost;
	s->m_pServerInfo = &g_ServerInfo;

	sockaddr_in* psa = s->m_hrHttpReq.psaRemoteHost;
	u_char* chAddr = (u_char*)&(psa->sin_addr.S_un.S_un_b);

	
	// ------------------------------------------
	// Accept loop
	// ------------------------------------------
	while(pChildThread->bValid != FALSE)
	{
		sprintf( szRemoteHost, "%u.%u.%u.%u", chAddr[0], chAddr[1], chAddr[2], chAddr[3] );

		// Set Accept socket.
		s->SetSocket( pChildThread->sock );

		// @RFC2616 sec.8.1 Persistent connection
		// pipeline was not supported.

		int nHttpStatus = 0;
		do
		{
			s->SetTimeOut(1);	// 1sec timeout
			// ----------------------------------------------------------------
			// Recive request...
			// ----------------------------------------------------------------
			DBGPRINT(pChildThread->hWnd, pChildThread->nMsgID,nHttpStatus, L">Recv");

			nHttpStatus = s->RecvRequest(pChildThread->pbBeQuick);

			if( nHttpStatus != 0 )	// 0==socket error
			{
				// ----------------------------------------------------------------
				// Send response
				// ----------------------------------------------------------------
				
				s->SetTimeOut(5);								// 5sec timeout

				pszCon = s->m_hashReqHeader.GetValueI(szConnection);

				if( pszCon == NULL )
				{
					pszCon = szKeepAlive;						// HTTP/1.1 Parmanent connection
				}

				// ----------------------------------------------------------------
				// Keep-Alive check
				// ----------------------------------------------------------------
				
				if(	!g_conf.m_nKeepAlive						// Global setting
					|| ( 0 == my_stricmp( pszCon, szClose )	)	// Connection: close
					|| *pChildThread->pbBeQuick	 				// Don't keep alive flag
					|| ( nHttpStatus >= 400 ))					// Must not Keep alive flag
				{
					bConnectionClose = TRUE;
				}
				else
				{
					bConnectionClose = FALSE;
				}

				DBGPRINT(pChildThread->hWnd, pChildThread->nMsgID,nHttpStatus, L"Send>");

				 nHttpStatus = s->SendResponse(nHttpStatus, bConnectionClose);
			}

			if( pChildThread->fnLogapi && nHttpStatus )
			{
				s->m_hashReqHeader.Set("remote_addr", szRemoteHost);
				pChildThread->fnLogapi( g_ServerInfo.pwszLogFile, g_ServerInfo.nLogSize, ( nHttpStatus == 999 ) ? 200 : nHttpStatus,  &(s->m_hashReqHeader) );
			}

			s->FreeMemory();

		}while( ( 200 <= nHttpStatus && nHttpStatus < 400 )
				&& ( pszCon != NULL )
				&& ( !bConnectionClose ) );
		
		// ------------------------------------------
		// Socket close
		// Connection: close
		// ------------------------------------------

		s->Close();

		// ------------------------------------------
		// Is terminate signal on?
		// ------------------------------------------
		if( pChildThread->bValid == FALSE )
			break;

		// ------------------------------------------
		// Suspend... resumed by thListen thread
		// ------------------------------------------
		DBGPRINT(pChildThread->hWnd, pChildThread->nMsgID,nHttpStatus, L"Sleep>");

		// To avoid deadlock between this thread and parent thread.
		SetThreadPriority( pChildThread->hThread, THREAD_PRIORITY_ABOVE_NORMAL );
		// So, I'm just going to sleep.
		pChildThread->bValid = FALSE;
		SuspendThread( pChildThread->hThread );
		// Wakeup by a parent thread.
		SetThreadPriority( pChildThread->hThread, THREAD_PRIORITY_NORMAL );
	}
	delete s;
	pChildThread->bValid = FALSE;
	DBGPRINT(pChildThread->hWnd, pChildThread->nMsgID,0, L"Exit");
	ExitThread(1);
}

//-----------------------------------------------------------------------------
//	@detail
//		Server thread
//	@param
//		THREADPACK
//	@return
//-----------------------------------------------------------------------------
void thListen(LPVOID pvParams)
{
	LISTEN_THREAD* ptpListenThread = (LISTEN_THREAD*)pvParams;
	CHILD_THREAD* pChildThread = (CHILD_THREAD*)malloc( sizeof(CHILD_THREAD) * g_conf.m_nStartServers );
	CHILD_THREAD* pCT;

	if( pChildThread == NULL )
	{
		SetDlgItemText( ptpListenThread->hWnd, IDC_MESSAGE, L"Memory error." );
		ExitThread(0);
	}
	
	static	int	bBeQuick = FALSE;
	CEHTTPLOGAPI fnLogapi;

	// load server settings
	g_ServerInfo.nPort				= g_conf.m_nListenPort;
	g_ServerInfo.pszCgiRootName		= g_conf.m_pszCgiRootName;
	g_ServerInfo.pszDocRootName		= g_conf.m_pszDocRootName;
	g_ServerInfo.pszSoftware		= g_conf.m_pszServerSoftware;
	g_ServerInfo.pszDefaultDocName	= g_conf.m_pszDefaultDocName;
	g_ServerInfo.pszAccessFileName	= g_conf.m_pszAccessFileName;
	g_ServerInfo.pwszLogFile		= malloc_char2wchar(g_conf.m_pszLogFile);
	g_ServerInfo.nLogSize			= g_conf.m_nLogSize;

	// server thread start
	ptpListenThread->bValid = TRUE;

	int ii = 0;

	// ------------------------------------------
	// load logapi
	// ------------------------------------------
	HMODULE hDLL = NULL;

	{
		WCHAR pwszLogDll[MAX_PATH] = {'\0'};
		GetCurrentDirW( pwszLogDll, MAX_PATH );

		wcscat( pwszLogDll, LOGAPIFILE );
		
		hDLL = ::LoadLibrary( pwszLogDll );

		if( hDLL != NULL )
		{
		
			fnLogapi = (CEHTTPLOGAPI)GetProcAddress( hDLL, FNC_LOG_API );
		}
		else
		{
			fnLogapi = NULL;
		}
	}


	// ------------------------------------------
	// Start listen
	// ------------------------------------------
	MySock s;

	if( SOCKET_ERROR == s.Listen(g_ServerInfo.nPort) )
	{
		SetDlgItemText( ptpListenThread->hWnd, IDC_MESSAGE, L"Listen Failed." );
		free( pChildThread );
		free( g_ServerInfo.pwszLogFile );
		ptpListenThread->bValid = FALSE;
		ExitThread(0);
	}
	SetDlgItemText(ptpListenThread->hWnd, IDC_MESSAGE, TEXT("Listen..."));
	
	// ------------------------------------------
	// Initialize enviroments for child thread
	// ------------------------------------------
	pCT = pChildThread;

	for( ii = 0; ii < g_conf.m_nStartServers; ii++, pCT++ )
	{
		pCT->bValid = FALSE;
		pCT->pbBeQuick = &bBeQuick;
		pCT->hWnd = ptpListenThread->hWnd;
		pCT->hThread = NULL;
		pCT->sock = INVALID_SOCKET;
		pCT->nMsgID = IDC_THREAD0 + ii;
		pCT->fnLogapi = fnLogapi;
	}

	// ------------------------------------------
	// Create child threads
	// ------------------------------------------
	pCT = pChildThread;

	for( ii = 0; ii < g_conf.m_nStartServers; ii++, pCT++ )
	{
		DWORD ThreadID;

		pCT->hThread = CreateThread(
			NULL, 0, (LPTHREAD_START_ROUTINE)thAccept, (LPVOID)pCT, CREATE_SUSPENDED, &ThreadID);

		if( pCT->hThread == NULL )
		{
			free( pChildThread );
			free( g_ServerInfo.pwszLogFile );
			SetDlgItemText(ptpListenThread->hWnd, IDC_MESSAGE, TEXT("Create thread failed."));
			ExitThread(0);
		}

		SetDlgItemText(ptpListenThread->hWnd, IDC_THREAD0 + ii, TEXT("Ready."));
	}
	
	// ------------------------------------------
	// Do Accept
	// ------------------------------------------
	while(ptpListenThread->bValid)
	{
		SOCKET client_socket;
		sockaddr_in saRemoteHost;

		pCT = pChildThread;

		s.SetTimeOut( 1 );

		if( s.IsReadalbe() > 0 )
		{
			// Serching free thread
			for( ii = 0; ii < g_conf.m_nStartServers; ii++, pCT++ )
			{
				Sleep(0);
				if( pCT->bValid == FALSE )
				{
					bBeQuick = FALSE;
					
					// Find free thread, so accept.
					// ACCEPT
					client_socket = s.Accept( &saRemoteHost );
					
					if( client_socket == INVALID_SOCKET )
						break;

					// Starting child thread.
					pCT->bValid = TRUE;
					pCT->sock = client_socket;
					memcpy( &(pCT->saRemoteHost), &saRemoteHost, sizeof(sockaddr_in) );
					ResumeThread(pCT->hThread);
					break;
				}
			}

			// Every thread is busy.
			if( ii >= g_conf.m_nStartServers )
			{
				client_socket = s.Accept( &saRemoteHost );
				if( client_socket != INVALID_SOCKET )
				{
					MySock sErrorSock(client_socket);
					sErrorSock.SendData( "HTTP/1.0 503 Server busy\r\n\r\n" );
					sErrorSock.Close();
				}
				// If child thread on Keep-Alive, stop it.
				bBeQuick = TRUE;
			}
		}
		else
		{
			// Timeout.
			// Save CPU power.
			Sleep(100);
		}
		continue;
	}

	// ------------------------------------------
	// Terminate child threads
	// ------------------------------------------
	//HANDLE*	phThreads = new HANDLE[g_conf.m_nStartServers];
	HANDLE*	phThreads = (HANDLE*)malloc( sizeof(HANDLE) * g_conf.m_nStartServers );

	for( ii = 0, pCT = pChildThread; ii < g_conf.m_nStartServers; ii++, pCT++ )
	{
		phThreads[ii] = pCT->hThread;
		pCT->bValid = FALSE;
		bBeQuick = TRUE;
		ResumeThread(pCT->hThread);
	}


	WaitForMultipleObjects( g_conf.m_nStartServers, phThreads, TRUE, 1000 );

	for( ii = 0, pCT = pChildThread; ii < g_conf.m_nStartServers; ii++, pCT++ )
	{
		if( pCT->hThread )
			CloseHandle( pCT->hThread );
	}

	if( hDLL )
	{
		FreeLibrary(hDLL);
	}

	//delete[] phThreads;
	//delete[] pChildThread;
	
	free( phThreads );
	free( pChildThread );
	free( g_ServerInfo.pwszLogFile );

	ptpListenThread->bValid = FALSE;

	ExitThread(1);

}
