#include <stdlib.h>
#include "httphelper.h"

// ----------------------------------------------------------------------------
// HELPER FUNCTIONS
// ----------------------------------------------------------------------------

int GetCurrentDirW(WCHAR* pwszDir, int nBufLen)
{
	DWORD dwLen;

	memset( pwszDir, 0, sizeof(WCHAR)*nBufLen );

	dwLen = GetModuleFileName( NULL, pwszDir, nBufLen );
	
#ifndef _WIN32_WCE
	WCHAR* pwSrc = pwszDir;
	WCHAR* pwDest = pwSrc;
	pwSrc += 2;
	
	do
	{
		*pwDest++ = *pwSrc;
	}while( *pwSrc++ != '\0' );
#endif
	if( dwLen == 0 )
		return 0;
	
	dwLen--;

	while( pwszDir[dwLen] != '\0' && pwszDir[dwLen] != '\\' )
	{
		dwLen--;
		if( dwLen == 0 )
			break;
	}

	pwszDir[dwLen] = '\0';

	return dwLen;
}


static char B64(char c)
{
	if( c == '=' ) return 0;
	const char b64[] ="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
	for( char ii = 0; b64[ii] != c && b64 != '\0'; ii++ );
	return ii;
}

char* Base64Decord(unsigned char *buf)
{
    char *p = (char*)buf;

    int l = strlen((const char*)buf);
    for (int ii=0,jj=0; ii<l; ii+=4, jj+=3)
	{
        p[jj]   =(char)(  (B64(buf[ii])           << 2) | ((B64(buf[ii+1]) & 0x30) >> 4) );
        p[jj+1] =(char)( ((B64(buf[ii+1]) & 0x0f) << 4) | ((B64(buf[ii+2]) & 0x3c) >> 2) );
        p[jj+2] =(char)( ((B64(buf[ii+2]) & 0x03) << 6) |  (B64(buf[ii+3]) & 0x3f)       );
    }
    p[jj] = '\0';
    return (p);
}

//-----------------------------------------------------------------------------
// '/' -> '\\'
void slash2bslash(char* str)
{
	if( str == NULL )
		return;

	for( ;*str;str++)
		if( *str == '/' ) *str = '\\';
}

//-----------------------------------------------------------------------------
// '/' <- '\\'
void bslash2slash(char* str)
{
	if( str == NULL )
		return;

	for( ;*str;str++)
		if( *str == '\\' ) *str = '/';
}

//-----------------------------------------------------------------------------
// char to wchar with malloc
WCHAR* malloc_char2wchar(char* pszStr)
{
	WCHAR* pwsRet = NULL;
	int nRet = 0;

	nRet = MultiByteToWideChar(CP_ACP,0,pszStr,-1,NULL,0);

	if( nRet == 0 )
	{
		return NULL;
	}

	pwsRet = (WCHAR*)malloc( nRet * sizeof(WCHAR) );

	if( 0 == MultiByteToWideChar(CP_ACP,0,pszStr,-1,pwsRet,nRet) )
	{
		free( pwsRet );
		return NULL;
	}

	return pwsRet;

}

//-----------------------------------------------------------------------------
// FindFirstFile MBCS
HANDLE FindFirstFileMBCS(char* lpFileName, LPWIN32_FIND_DATA lpFindFileData )
{
	TCHAR	tszFile[MAX_PATH];

	if( 0 == MultiByteToWideChar(CP_ACP,0,lpFileName,-1,tszFile,MAX_PATH) )
	{
		return INVALID_HANDLE_VALUE;
	}

	return FindFirstFile(tszFile, lpFindFileData );
}

//-----------------------------------------------------------------------------
// Convert a char to lowercase.
int mytolower( int c )
{
	return (( c >= 'A' && c <= 'Z' ) ? c += ('a'-'A') : c);
}

//-----------------------------------------------------------------------------
// Convert a string to lowercase.
char *my_strlwr( char *string )
{
	if( !string )
		return NULL;

	while( *string )
	{
		*string = mytolower( *string );
	}
	return string;
}

//-----------------------------------------------------------------------------
// Compare strings without regard to case
int my_stricmp( const char *str1, const char *str2 )
{
	do
	{
		if( mytolower(*str1) != mytolower(*str2) )
			return mytolower(*str2) - mytolower(*str1);
	}while( *str1++ && *str2++ );

	return 0;
}

//-----------------------------------------------------------------------------
// Compare characters of two strings without regard to case.
int my_strnicmp( const char *str1, const char *str2, size_t n )
{
	do
	{
		if( !n-- ) return 0;
		if( mytolower(*str1) != mytolower(*str2) )
			return mytolower(*str2) - mytolower(*str1);
	}while( *str1++ && *str2++ );

	return 0;
}

//-----------------------------------------------------------------------------
// Duplicate strings. 
char* my_strdup( const char* strSrc )
{
	int nLen;
	char* pNew;

	if( strSrc == NULL ) return NULL;

	nLen = strlen(strSrc);
	//pNew = new char[nLen + 1];
	pNew = (char*)malloc(nLen+1);
	
	if( pNew )
	{
		strcpy( pNew, strSrc );
	}
	return pNew;
}
