/* =========================================================================
	This extention is part of IshiSoft CEHTTP.
	------------------------------------------------------------------------
	author: yu-ishi@ya2.so-net.ne.jp
			http://www001.upp.so-net.ne.jp/ishi/index.html
  ========================================================================== */
#include "stdafx.h"
#include <stdio.h>	// for PCHTTP
#include <stdlib.h>	// for PCHTTP
#include "..\cehttp2\MyHash.h"

CRITICAL_SECTION cs;

#define MAX_LOG_SIZE	

char* pNullString = "-";
char* pszMonth[12] =
{
	"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

static char* GetValueAvoidNull( MyHash* pHashHeaders, char* pszKey );

//-----------------------------------------------------------------------------
//	@detail
//		Create logfile extention
//		work on CEHTTP directory.
//	@param
//		WCHAR* pwszLogFile		// File to create
//		int nLogSize			// Limit of logfile
//		int nHttpStatus			// http status
//		MyHash* pHashHeaders	// headers
//	@return
//	@comment
//-----------------------------------------------------------------------------
int fnCehttpLog( WCHAR* pwszLogFile, int nLogSize, int nHttpStatus, MyHash* pHashHeaders )
{
	HANDLE hFile = INVALID_HANDLE_VALUE;
	// TIME_ZONE_INFORMATION tzi;
	DWORD dwWritten;
	char szBuf[512];

	// ------------------------------------------------------------------------
	EnterCriticalSection( &cs );

	// GetTimeZoneInformation( &tzi );
	// long biashour = tzi.Bias / 60;
	// long biasmin = (unsigned)(tzi.Bias % 60);
	SYSTEMTIME st;
	GetLocalTime(&st);

	char* pRemoteHost	= GetValueAvoidNull( pHashHeaders, "remote_addr" );
	char* pRequestLine	= GetValueAvoidNull( pHashHeaders, "request_line" );
	char* pSendBytes	= GetValueAvoidNull( pHashHeaders, "send_bytes" );
	char* pReferer		= GetValueAvoidNull( pHashHeaders, "referer" );
	char* pUserAgent	= GetValueAvoidNull( pHashHeaders, "user-agent" );
	char* pAuthName		= GetValueAvoidNull( pHashHeaders, "authentication_user" );

	if( strlen(pRemoteHost)
		+ strlen(pRequestLine)
		+ strlen(pReferer)
		+ strlen(pUserAgent)
		+ strlen(pAuthName) > 400 )
	{
		LeaveCriticalSection( &cs );
		return 0;
	}

	// ------------------------------------------------------------------------
	// create file
	// ------------------------------------------------------------------------

	hFile = CreateFile( pwszLogFile, GENERIC_WRITE, 0, NULL, OPEN_ALWAYS, FILE_FLAG_RANDOM_ACCESS, NULL);
	
	if( hFile == INVALID_HANDLE_VALUE )
	{
		LeaveCriticalSection( &cs );
		return 0;
	}

	SetFilePointer( hFile, 0, 0, FILE_END );

	sprintf( szBuf, 
		"%s - %s [%02d/%s/%4d:%02d:%02d:%02d +900] \"%s\" %d %s \"%s\" \"%s\"\n"
		, pRemoteHost
		, pAuthName
		, st.wDay, pszMonth[st.wMonth-1], st.wYear
		, st.wHour, st.wMinute, st.wSecond
		, pRequestLine
		, nHttpStatus
		, pSendBytes
		, pReferer
		, pUserAgent );
	WriteFile( hFile, szBuf,strlen(szBuf), &dwWritten, NULL );

	DWORD dwFileSize = GetFileSize( hFile, NULL ); 

	// ------------------------------------------------------------------------
	// close file
	// ------------------------------------------------------------------------

	CloseHandle( hFile );
	hFile = INVALID_HANDLE_VALUE;

	// ------------------------------------------------------------------------
	// change backup if necessary
	// ------------------------------------------------------------------------

	if( dwFileSize > (DWORD)nLogSize )
	{
		int nLen = wcslen( pwszLogFile );
		WCHAR* pBackUp = (WCHAR*)malloc( nLen + sizeof(WCHAR) );
		
		if( pBackUp )
		{
			wcscpy( pBackUp, pwszLogFile );
			pBackUp[nLen-1] = '_';
			DeleteFile( pBackUp );
			MoveFile( pwszLogFile, pBackUp );
			free( pBackUp );
		}
	}

	// ------------------------------------------------------------------------
	LeaveCriticalSection( &cs );
	return 1;
}

static char* GetValueAvoidNull( MyHash* pHashHeaders, char* pszKey )
{
	char* p =  pHashHeaders->GetValueI( pszKey );
	if( p == NULL )
		return pNullString;
	return p;
}


BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch( ul_reason_for_call )
	{
	case DLL_PROCESS_ATTACH:
		{
			InitializeCriticalSection(&cs);
			break;
		}
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		break;
	case DLL_PROCESS_DETACH:
		{
			DeleteCriticalSection( &cs );
			break;
		}
    }

    return TRUE;
}

